#pragma once

//Copyright (C) cocosimulator.org 2014
//generated by AmsterCHEM's COM CAPE-OPEN Wizard

//BSTR helper routines

__inline bool SameString(const OLECHAR *str1,const OLECHAR *str2) {
	bool res=false;
	if (str1) {
		if (str2) {
			res=(lstrcmpiW(str1,str2)==0);
		} else if (*str1==0) {
			res=true;
		}
	} else if (!str2) {
		res=true;
	} else if (*str2==0) {
		res=true;
	}
	return res;
}
__inline int StringLen(const OLECHAR *str) {
	return (str)?lstrlenW(str):0;
}


//BSTR wrapper class

class CBStr {

	BSTR b;

	public:

	CBStr() {
		b=NULL;
	}

	CBStr(const OLECHAR *txt) {
		b=SysAllocString(txt);
	}

	CBStr(const wstring &txt) {
		b=(txt.empty())?NULL:SysAllocString(txt.c_str());
	}

	CBStr(const CBStr &other) {
		//copy 
		b=SysAllocString(other.b);
	}

	~CBStr() {
		if (b) SysFreeString(b);
	}
	
	bool operator !() const {
		return (!b);
	}

	operator BSTR() const 
	{	//cast
		return b;
	}

	CBStr& operator=(const BSTR other) {
		if (b) SysFreeString(b);
		if (other) b=SysAllocString(other); else b=NULL;
		return *this;
	}

	CBStr& operator=(CBStr &other) {
		//for this assignment we presume we are not longer interested in the original value
		// as presumably this is a return value from something like CVariant::GetStringAt
		// hence, we steal the content
		if (b) SysFreeString(b);
		b=other.b;
		other.b=NULL;
		return *this;
	}

	CBStr& operator=(const OLECHAR *txt) {
		if (b) SysFreeString(b);
		if (txt) b=SysAllocString(txt); else b=NULL;
		return *this;
	}

	CBStr& operator=(wstring str) {
		if (b) SysFreeString(b);
		b=(str.empty())?NULL:SysAllocString(str.c_str());
		return *this;
	}

	bool operator==(const OLECHAR *txt) const {
		return SameString(b,txt);
	}

	bool operator==(wstring str) const {
		return SameString(b,str.c_str());
	}

	bool operator!=(const OLECHAR *txt) const {
		return !SameString(b,txt);
	}

	bool operator!=(wstring str) const {
		return !SameString(b,str.c_str());
	}

	BSTR* operator&() {
		//presume we are being passed to a function as an [out, retval] argument
		ASSERT(!b,"BSTR value is non-null when passing to external function");
		return &b;
	}

	size_t size() {
		return (b)?lstrlenW(b):0;
	}

	bool empty() {
		return (b)?(*b==0):true;
	}

	void clear() {
		if (b) {
			SysFreeString(b);
			b=NULL;
		}
	}

	//attach and detach to take and release ownership

	const BSTR Attach(BSTR to) {
		b=to;
		return b;
	}

	BSTR Detach() {
		BSTR res=b;
		b=NULL;
		return res;
	}

};

//BSTR constant wrapper class

class CBStrConst {

	BSTR b;

	//no assignment
	void operator=(const CBStrConst &other);

	public:

	CBStrConst(const wchar_t *str) {
		b=(str)?SysAllocString(str):NULL;
	}

	CBStrConst(const wstring &str) {
		b=(str.empty())?NULL:SysAllocString(str.c_str());
	}

	operator BSTR() const 
	{	//cast
		return b;
	}

};
